function J = MPC_objective(u,P,V)
%% Cost function for MPC of marine vessel power system
% Inputs:
%   u       optimization variable
%   P       controller parameters (see fmincon_wrapper.m)
%   V       controller variables (see fmincon_wrapper.m)
% Output:
%   J:      objective function cost

% Copyright (c) 2018, Andreas Reason Dahl
% All rights reserved. Published under MIT. See LICENSE.txt.
% Please include the reference given in reference.txt whenever using this.

% Predict x(k=1) given current load and manipulated variables
P0 = V.P_hat(1:P.N)';
u0 = V.U_opt_prev(1:P.N);

xk = spmDT(u0,V.x_hat,P,P0);

J = 0;

for k = 1:P.N % Prediction steps
    % Predict x(k+1)
    Pk = V.P_hat(4*k+1:4*k+4)';
    uk = u(4*k-3:4*k);
    xk = spmDT(uk,xk,P,Pk);
    
    % State deviation cost
     J = J + (xk-P.x_ref)' * P.Q * (xk-P.x_ref);
    
    % Manipulated variables rate of change cost
    if k==1
        J = J + (uk-u0)' * P.R(:,:,P.simRun) * (uk-u0);
    else
        J = J + (uk-u(4*k-7:4*k-4))' * P.R(:,:,P.simRun) * (uk-u(4*k-7:4*k-4));
    end
    
    % Additional term to avoid deviation from optimal setpoint = 0.8
    S_weight = 0.01;
    S = diag(S_weight*[1 1 1 1]);
    
    J = J + (uk-P.u_0)' * S * (uk-P.u_0);
    
end
end