function x_dot = spmCT(t,u,x,P,Pk) 
%% Continuous-time structure preserving model
% Inputs:
%   u       control input
%   x       state vector
%   P       controller parameters (see fmincon_wrapper.m)
%   Pk      load vector P(k)
% Output:
%   x_dot   state derivative

% Copyright (c) 2018, Andreas Reason Dahl
% All rights reserved. Published under MIT. See LICENSE.txt.
% Please include the reference given in reference.txt whenever using this.

%% Calculate SPM
f_flow = P.B*(P.B_sus.*sin(-P.B'*x(1:8)));       % Power flow
f_gov  = u-(x(9:12)-1)/0.05;                     % Governor
f_COI  = (ones(1,4)*P.M*x(9:12))/sum(diag(P.M)); % Centre of inertia

x_dot  = [P.D\(P.fU*f_flow + Pk)
          (x(9:12)-f_COI)*P.omega_B
          P.M\(f_gov + sum(P.S)./P.S.*P.fL*f_flow./x(9:12))];
end