function J = MPC_objective(u,P,V)
%% Cost function for MPC of marine vessel power system
% Inputs:
%   u       optimization variable
%   P       controller parameters (see fmincon_wrapper.m)
%   V       controller variables (see fmincon_wrapper.m)
% Output:
%   J:      objective function cost

% Copyright (c) 2018, Andreas Reason Dahl
% All rights reserved. Published under MIT. See LICENSE.txt.
% Please include the reference given in reference.txt whenever using this.

% Predict x(k=1) given current load and manipulated variables
P0 = V.P_hat(1:P.N)';
u0 = V.U_opt_prev(1:5);

xk = spmDT(u0,V.x_hat,P,P0,V.P_ref_batt);

J = 0;

for k = 1:P.N % Prediction steps
    % Predict x(k+1)
    Pk = V.P_hat(4*k+1:4*k+4)';
    uk = u(5*k-4:5*k);
    xk = spmDT(uk,xk,P,Pk,V.P_ref_batt);
    
    % State deviation cost
    J = J + (xk(1:12)-P.x_ref)' * P.Q * (xk(1:12)-P.x_ref);
        
    % Manipulated variables rate of change cost
    if k==1
        J = J + (uk(1:4)-u0(1:4))' * P.R(:,:,P.simRun) * (uk(1:4)-u0(1:4));
    else
        J = J + (uk(1:4)-u(5*k-9:5*k-6))' * P.R(:,:,P.simRun) * (uk(1:4)-u(5*k-9:5*k-6));
    end
    
    % Additional term to avoid deviation from optimal setpoint = 0.8
    S_weight = 0.01;
    S = diag(S_weight*[1 1 1 1]);
    
    J = J + (uk(1:4)-P.u_0)' * S * (uk(1:4)-P.u_0);
    
    % Minimize SOC deviation
    SOC_weight = 100;
    SOC = xk(13);
    SOC_desired = 0.5;
    J = J + SOC_weight*(SOC - SOC_desired)^2;
    
end
end