%% Calculate optimal battery power for the strategic loading

function P_batt_opt = strategic_optimization(P,P_load,P_gen_max,gens_on)

% Output of the function is the optimal charge and discharge power

% Calculate available generator power
total_gen_power = [P.gen1 P.gen2 P.gen3 P.gen4] * gens_on;

% Distribute evenly on each genset
scaling_factor = P_gen_max/total_gen_power;
P_load = P_load * scaling_factor;

% Objective function
obj = @(x) F_objective(x,P_load,P_gen_max);

P_batt_c_max = 2e6 * scaling_factor;
P_batt_d_max = 2e6 * scaling_factor;

P_gc_min = P_load;
P_gc_max = P_load + P_batt_c_max;

P_gd_min = P_load - P_batt_d_max;
P_gd_max = P_load;

LB = [P_gc_min, P_gd_min];
UB = [P_gc_max, P_gd_max];

% Initial point is at the furthest point, with highest battery usage
x0 = [P_gc_max, P_gd_min];

options = optimoptions('fmincon','Algorithm','interior-point','MaxFunEvals', 400000,'Display','none');

% Include constraints on engine (0 < P < P_gen_max)
A = [ 1  0 ;
      0 -1 ];

b = [ P_gen_max ;
          0 ];

% Solve optimization, where x_opt = [P_gc, P_gd]
x_opt = fmincon(obj, x0, A, b, [],[],LB,UB,[],options);

P_gc_opt = x_opt(1);
P_gd_opt = x_opt(2);

P_batt_c_opt = P_gc_opt - P_load;
P_batt_d_opt = P_load - P_gd_opt;

% Output
P_batt_opt = [P_batt_c_opt, P_batt_d_opt];

end